@@ -1,6 +1,8 @@
 /****************************************************************************
  * apps/nshlib/nsh_timcmds.c
  *
+ * SPDX-License-Identifier: Apache-2.0
+ *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.  The
@@ -25,6 +27,7 @@
 #include <nuttx/config.h>
 
 #include <stdlib.h>
+#include <string.h>
 #include <strings.h>
 #include <unistd.h>
 #include <time.h>
@@ -434,6 +437,7 @@ int cmd_date(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
   return ret;
 
 errout:
+  optind = 0;
   nsh_error(vtbl, errfmt, argv[0]);
   return ERROR;
 }
@@ -544,3 +548,64 @@ int cmd_timedatectl(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
   return ret;
 }
 #endif
+
+#ifndef CONFIG_NSH_DISABLE_WATCH
+int cmd_watch(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
+{
+  char buffer[CONFIG_NSH_LINELEN];
+  int interval = 2;
+  int count = -1;
+  FAR char *cmd;
+  int option;
+  int ret;
+  int i;
+
+  while ((option = getopt(argc, argv, "n:c:")) != ERROR)
+    {
+      switch (option)
+        {
+          case 'n':
+            interval = atoi(optarg);
+            break;
+
+          case 'c':
+            count = atoi(optarg);
+            break;
+
+          default:
+            nsh_error(vtbl, g_fmtarginvalid, argv[0]);
+            return ERROR;
+        }
+    }
+
+  if (optind < argc)
+    {
+      cmd = argv[optind];
+    }
+  else
+    {
+      nsh_error(vtbl, g_fmtarginvalid, argv[0]);
+      return ERROR;
+    }
+
+  if (count < 0)
+    {
+      count = INT_MAX;
+    }
+
+  for (i = 0; i < count; i++)
+    {
+      strlcpy(buffer, cmd, CONFIG_NSH_LINELEN);
+      ret = nsh_parse(vtbl, buffer);
+      if (ret < 0)
+        {
+          nsh_error(vtbl, g_fmtcmdfailed, argv[0], cmd, NSH_ERRNO);
+          return ERROR;
+        }
+
+      sleep(interval);
+    }
+
+  return OK;
+}
+#endif
