@@ -1,8 +1,9 @@
 /****************************************************************************
  * apps/system/cu/cu_main.c
  *
- *   Copyright (C) 2014 sysmocom - s.f.m.c. GmbH. All rights reserved.
- *   Author: Harald Welte <hwelte@sysmocom.de>
+ * SPDX-License-Identifier: BSD-3-Clause
+ * SPDX-FileCopyrightText: 2014 sysmocom - s.f.m.c. GmbH. All rights reserved.
+ * SPDX-FileContributor: Harald Welte <hwelte@sysmocom.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -57,9 +58,9 @@
 #include "cu.h"
 
 #ifdef CONFIG_SYSTEM_CUTERM_DISABLE_ERROR_PRINT
-# define cu_error(...)
+#  define cu_error(...)
 #else
-# define cu_error(...) dprintf(STDERR_FILENO, __VA_ARGS__)
+#  define cu_error(...) dprintf(STDERR_FILENO, __VA_ARGS__)
 #endif
 
 /****************************************************************************
@@ -110,7 +111,11 @@ static FAR void *cu_listener(FAR void *parameter)
           break;
         }
 
-      write(STDOUT_FILENO, &ch, 1);
+      rc = write(STDOUT_FILENO, &ch, 1);
+      if (rc <= 0)
+        {
+          break;
+        }
     }
 
   /* Won't get here */
@@ -133,7 +138,7 @@ static int set_termios(FAR struct cu_globals_s *cu, int nocrlf)
   int ret;
   struct termios tio;
 
-  if (isatty(cu->devfd))
+  if (isatty(cu->devfd) && isatty(cu->stdfd))
     {
       tio = cu->devtio;
 
@@ -182,38 +187,31 @@ static int set_termios(FAR struct cu_globals_s *cu, int nocrlf)
           cu_error("set_termios: ERROR during tcsetattr(): %d\n", errno);
           return ret;
         }
+
+      /* Let the remote machine to handle all crlf/echo except Ctrl-C */
+
+      tio = cu->stdtio;
+
+      tio.c_iflag = 0;
+      tio.c_oflag = 0;
+      tio.c_lflag &= ~ECHO;
+
+      ret = tcsetattr(cu->stdfd, TCSANOW, &tio);
+      if (ret)
+        {
+          cu_error("set_termios: ERROR during tcsetattr(): %d\n", errno);
+          return ret;
+        }
     }
 
-  /* Let the remote machine to handle all crlf/echo except Ctrl-C */
-
-  if (cu->stdfd >= 0)
-  {
-    tio = cu->stdtio;
-
-    tio.c_iflag = 0;
-    tio.c_oflag = 0;
-    tio.c_lflag &= ~ECHO;
-
-    ret = tcsetattr(cu->stdfd, TCSANOW, &tio);
-    if (ret)
-      {
-        cu_error("set_termios: ERROR during tcsetattr(): %d\n", errno);
-        return ret;
-      }
-  }
-
   return 0;
 }
 
 static void retrieve_termios(FAR struct cu_globals_s *cu)
 {
-  if (isatty(cu->devfd))
+  if (isatty(cu->devfd) && isatty(cu->stdfd))
     {
       tcsetattr(cu->devfd, TCSANOW, &cu->devtio);
-    }
-
-  if (cu->stdfd >= 0)
-    {
       tcsetattr(cu->stdfd, TCSANOW, &cu->stdtio);
     }
 }
