@@ -1,6 +1,8 @@
 /****************************************************************************
  * apps/system/uorb/uORB/uORB.c
  *
+ * SPDX-License-Identifier: Apache-2.0
+ *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.  The
@@ -370,7 +372,7 @@ void orb_info(FAR const char *format, FAR const char *name,
   vaf.va  = (va_list *)data;
 
   lib_stdoutstream(&stdoutstream, stdout);
-  lib_sprintf(&stdoutstream.common, "%s(now:%" PRIu64 "):%pB",
+  lib_sprintf(&stdoutstream.common, "%s(now:%" PRIu64 "):%pB\n",
               name, orb_absolute_time(), &vaf);
 }
 
