@@ -1,6 +1,8 @@
 /****************************************************************************
  * apps/testing/drivertest/drivertest_rtc.c
  *
+ * SPDX-License-Identifier: Apache-2.0
+ *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.  The
@@ -39,6 +41,7 @@
 #include <stdint.h>
 #include <cmocka.h>
 #include <syslog.h>
+#include <getopt.h>
 #include <nuttx/timers/rtc.h>
 #include <nuttx/clock.h>
 
@@ -229,7 +232,7 @@ static void add_timeout(struct rtc_time * rtc_tm, const int delay)
   timesp = timegm((struct tm *)rtc_tm);
   timesp += delay;
 
-  tm = localtime(&timesp);
+  tm = gmtime(&timesp);
   rtc_tm->tm_sec = tm->tm_sec;
   rtc_tm->tm_min = tm->tm_min;
   rtc_tm->tm_hour = tm->tm_hour;
