@@ -1,6 +1,8 @@
 /****************************************************************************
  * apps/testing/monkey/monkey_type.h
  *
+ * SPDX-License-Identifier: Apache-2.0
+ *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.  The
@@ -98,6 +100,8 @@ struct monkey_config_s
 {
   struct
   {
+    int x_offset;
+    int y_offset;
     int hor_res;
     int ver_res;
   } screen;
