@@ -1,6 +1,8 @@
 /****************************************************************************
  * apps/testing/sensortest/sensortest.c
  *
+ * SPDX-License-Identifier: Apache-2.0
+ *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.  The
@@ -350,7 +352,7 @@ int main(int argc, FAR char *argv[])
       goto name_err;
     }
 
-  snprintf(devname, PATH_MAX, DEVNAME_FMT, name);
+  snprintf(devname, sizeof(devname), DEVNAME_FMT, name);
   fd = open(devname, O_RDONLY | O_NONBLOCK);
   if (fd < 0)
     {
