@@ -1,24 +1,28 @@
 /****************************************************************************
  * apps/testing/testsuites/kernel/fs/cases/fs_fsync_test.c
- * Copyright (C) 2020 Xiaomi Corporation
  *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * SPDX-License-Identifier: Apache-2.0
  *
- *      http://www.apache.org/licenses/LICENSE-2.0
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.  The
+ * ASF licenses this file to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance with the
+ * License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
+ * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+ * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
+ * License for the specific language governing permissions and limitations
+ * under the License.
+ *
+ ****************************************************************************/
 
 /****************************************************************************
  * Included Files
  ****************************************************************************/
-
 #include <nuttx/config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,8 +44,10 @@
  ****************************************************************************/
 
 #define TESTFILE "FsyncTestFile"
-#define BUF "testData123#$%*-=/ sdafasd37575sasdfasdf3563456345" \
-            "63456ADSFASDFASDFQWREdf4as5df4as5dfsd ###"
+#define BUF                                                             \
+  "testData123#$%*-=/ "                                                 \
+  "sdafasd37575sasdfasdf356345634563456ADSFASDFASDFQWREdf4as5df4as5dfs" \
+  "d ###"
 
 /****************************************************************************
  * Public Functions
@@ -53,6 +59,8 @@
 
 void test_nuttx_fs_fsync01(FAR void **state)
 {
+  /* clock_t start, finish; */
+
   int fd;
   int rval;
   int ret;
@@ -97,7 +105,9 @@ void test_nuttx_fs_fsync02(FAR void **state)
   ssize_t writen = 0;
   struct statfs statfsbuf;
   struct fs_testsuites_state_s *test_state;
+  struct mallinfo mem_info;
 
+  memset(&mem_info, 0, sizeof(mem_info));
   test_state = (struct fs_testsuites_state_s *)*state;
 
   /* delete test file */
@@ -114,10 +124,23 @@ void test_nuttx_fs_fsync02(FAR void **state)
 
   ret = fstatfs(fd, &statfsbuf);
   assert_int_equal(ret, 0);
-
+#ifdef CONFIG_ARCH_SIM
   bufsize = statfsbuf.f_bsize;
+#else
+  get_mem_info(&mem_info);
+  if (mem_info.mxordblk < statfsbuf.f_bsize)
+    {
+      bufsize = mem_info.mxordblk - 16;
+    }
 
-  syslog(LOG_INFO, "the bsize = %" PRIdPTR "\n", statfsbuf.f_bsize);
+  else
+    {
+      bufsize = statfsbuf.f_bsize;
+    }
+
+#endif
+  syslog(LOG_INFO, "the fbsize = %"PRIu64",buffer size=%d\n",
+        statfsbuf.f_bsize, bufsize);
 
   /* malloc memory */
 
